<?php

namespace Drupal\cortex_general_settings\EventSubscriber;

use Drupal\Core\Config\ConfigEvents;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Config\StorageTransformEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Config Export/Import Subscriber for Cortex general settings.
 */
class CortexGeneralSettingsConfigSyncSubscriber implements EventSubscriberInterface {

  /**
   * The active config storage.
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $activeStorage;

  /**
   * The sync config storage.
   *
   * @var \Drupal\Core\Config\StorageInterface
   */
  protected $syncStorage;

  /**
   * DirectoriesConfigSubscriber constructor.
   *
   * @param \Drupal\Core\Config\StorageInterface $config_storage
   *   The config active storage.
   * @param \Drupal\Core\Config\StorageInterface $sync_storage
   *   The sync config storage.
   */
  public function __construct(StorageInterface $config_storage, StorageInterface $sync_storage) {
    $this->activeStorage = $config_storage;
    $this->syncStorage = $sync_storage;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ConfigEvents::STORAGE_TRANSFORM_IMPORT => ['onConfigTransformImport', 100],
    ];
  }

  /**
   * Reacts to the ConfigEvents::STORAGE_TRANSFORM_IMPORT event.
   *
   * @param \Drupal\Core\Config\StorageTransformEvent $event
   *   The config storage transform event.
   */
  public function onConfigTransformImport(StorageTransformEvent $event) {
    $transformation_storage = $event->getStorage();
    $destination_storage = $this->activeStorage;

    $config = $destination_storage->listAll('cortex_general_settings.');
    foreach ($config as $config_name) {
      if (!$transformation_storage->exists($config_name)) {
        $transformation_storage->write($config_name, $destination_storage->read($config_name));
      }
    }
  }

}
