<?php

declare(strict_types=1);

namespace Drupal\cortex_general_settings\EventSubscriber;

use Drupal\config_translation\Event\ConfigMapperPopulateEvent;
use Drupal\config_translation\Event\ConfigTranslationEvents;
use Drupal\cortex_general_settings\CortexGeneralSettingsPluginManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * @todo Add description for this subscriber.
 */
final class CortexGeneralSettingsTranslationSubscriber implements EventSubscriberInterface {

  /**
   * The general settings manager.
   *
   * @var \Drupal\cortex_general_settings\GeneralSettingsManagerInterface
   */
  protected $generalSettingsManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(CortexGeneralSettingsPluginManager $generalSettingsManager) {
    $this->generalSettingsManager = $generalSettingsManager;
  }

  /**
   * Kernel response event handler.
   */
  public function addConfigNames(ConfigMapperPopulateEvent $event): void {
    $generalSettingsManager = \Drupal::service('plugin.manager.cortex_general_settings');
    $mapper = $event->getMapper();
    if ($mapper->getBaseRouteName() === 'cortex_general_settings.general') {
      $generalSettingPlugins = $generalSettingsManager->getDefinitions();
      foreach ($generalSettingPlugins as $pluginId => $pluginDefinition) {
        $plugin = $generalSettingsManager->createInstance($pluginId);
        if ($plugin->hasTranslationAccess()) {
          $mapper->addConfigName($pluginDefinition['configName']);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ConfigTranslationEvents::POPULATE_MAPPER => ['addConfigNames'],
    ];
  }

}
