<?php

declare(strict_types=1);

namespace Drupal\cortex_general_settings\Form;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\cortex_general_settings\CortexGeneralSettingsPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Cortex general settings form.
 */
class CortexGeneralSettingsPluginForm extends FormBase {

  /**
   * The general settings manager.
   *
   * @var \Drupal\cortex_general_settings\GeneralSettingsManagerInterface
   */
  protected $generalSettingsManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(CortexGeneralSettingsPluginManager $generalSettingsManager) {
    $this->generalSettingsManager = $generalSettingsManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.cortex_general_settings'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cortex_general_settings_cortex_general_settings_plugin';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $generalSettingPlugins = $this->generalSettingsManager->getDefinitions();
    $pluginId = $form_state->getBuildInfo()['args'][1];
    $pluginDefinition = $generalSettingPlugins[$pluginId];

    $form = [];

    $form['save'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save @label', ['@label' => $pluginDefinition['label']]),
      '#submit' => [[$this, 'submitForm']],
      '#weight' => 100,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // $form_state->setRebuild(TRUE);
    $parents = $form_state->getTriggeringElement()['#parents'];
    array_pop($parents);
    $values = NestedArray::getValue($form_state->getValues(), $parents);

    // Remove clean values.
    foreach ($form_state->getCleanValueKeys() as $key) {
      if (!empty($values[$key])) {
        unset($values[$key]);
      }
    }
    // Remove button values.
    foreach ($form_state->getButtons() as $button) {
      $button_parents = $button['#array_parents'];
      NestedArray::unsetValue($values, $button_parents);
    }
    $pluginId = $form_state->getBuildInfo()['args'][1];

    $plugin = $this->generalSettingsManager->createInstance($pluginId);
    $plugin->submitSettingsForm($values);
  }

}
