<?php

declare(strict_types=1);

namespace Drupal\cortex_general_settings\Form;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\cortex_general_settings\CortexGeneralSettingsPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a Cortex general settings form.
 */
final class CortexSettingsForm extends FormBase {
  use DependencySerializationTrait;

  /**
   * The general settings manager.
   *
   * @var \Drupal\cortex_general_settings\GeneralSettingsManagerInterface
   */
  protected $generalSettingsManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(CortexGeneralSettingsPluginManager $generalSettingsManager) {
    $this->generalSettingsManager = $generalSettingsManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.cortex_general_settings'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cortex_general_settings_cortex_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $generalSettingPlugins = $this->generalSettingsManager->getDefinitions();
    $form['#submit'][] = [$this, 'submitForm'];
    $form['cortex_general_settings'] = [
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-company-details',
    ];

    foreach ($generalSettingPlugins as $pluginId => $pluginDefinition) {
      $plugin = $this->generalSettingsManager->createInstance($pluginId);
      $pluginForm = $plugin->settingsForm($form_state, $form);
      if ($pluginForm) {
        $pluginForm[$pluginId]['#group'] = 'cortex_general_settings';
        $pluginForm[$pluginId]['#tree'] = TRUE;
        $pluginForm[$pluginId]['#type'] = 'details';
        $pluginForm[$pluginId]['#title'] = $pluginDefinition['label'];
        if (!isset($pluginForm[$pluginId]['#weight'])) {
          $pluginForm[$pluginId]['#weight'] = 0;
        }
        $pluginForm[$pluginId]['description'] = [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#weight' => -100,
        [
          '#type' => 'html_tag',
          '#tag' => 'h3',
          '#value' => $pluginDefinition['label'],
        ],
        ];
        $pluginForm[$pluginId]['plugin_id'] = [
          '#type' => 'hidden',
          '#value' => $pluginId,
        ];

        $form['#cache'] = ['max-age' => 0];

        $form += $pluginForm;
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

  }

}
